/************************************************************************************/
/*	Copyright 2014 Cosmic Warpigs													*/
/*																					*/
/*	This file is part of Chipotle Warfare.											*/
/*																					*/
/*   Chipotle Warfare is free software: you can redistribute it and/or modify		*/
/*    it under the terms of the GNU General Public License as published by			*/
/*    the Free Software Foundation, either version 3 of the License, or				*/
/*		(at your option) any later version.											*/
/*																					*/
/*    Chipotle Warfare is distributed in the hope that it will be useful,			*/
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of				*/
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the					*/
/*    GNU General Public License for more details.									*/
/*																					*/
/*    You should have received a copy of the GNU General Public License				*/
/*    along with Chipotle Warfare.  If not, see <http://www.gnu.org/licenses/>.		*/
/*																					*/
/************************************************************************************/

#include "cpcrslib.h"
#include "cpcwyzlib.h"
#include "sprites.h"
#include "music.h"
#include "string.h"
#include "cosmicPathfinding.h"
#include <time.h>
#include <stdlib.h>

#define GAME_STATE_INITIAL 2
#define GAME_STATE_MENU    2
#define GAME_STATE_GAME    4
#define GAME_STATE_EXIT    5
#define GAME_STATE_WIN     6
#define GAME_STATE_LOSE    7
#define GAME_STATE_HOWTO   8
#define GAME_STATE_INTRO   9
#define GAME_STATE_CREDITS 10

#define SCORE_GOAL 8


#define MILLASMAX	40
#define TIMEMAX		800

#define PLAYER	0
#define ENEMY	1
#define SHOOT	2

#define TILE_X 	4
#define TILE_Y 	10
 
#define IZQ  				0
#define DER 				1
#define ARRIBA  			2
#define ABAJO  				3
#define SPACE   			4
#define KEY_1   			5
#define KEY_ESC    			6 
#define KEY_2				7
#define KEY_3				8

#define DIR_ABAJO  	0
#define DIR_ARRIBA 	1
#define DIR_DER 	2
#define DIR_IZQ 	3
#define STOP		4

#define ELAPSED_TIME	50

char state;
int test = 0;
char actualLevel;
char prevLevel;
char enemyCont;

unsigned char *pl_frames[4][3];
unsigned char *enemy_frames[4][3];


struct sprite  		
{
	char *frame;		
	char anim_num;
	char type;
	char width, height;
	char vida;
	char shoot;
	unsigned char x, y;	
	unsigned char ox, oy;
	unsigned char actx, acty;
	unsigned char modulo;
	char steps;
	int address;
	char dir;
	char dir_prev;
	int nodosRecorridos;
	char millas;
	int timestop;


};

struct sprite player, enemy, /*enemy2, enemy3,*/ shoot, door;

const struct Node nodosRespawn[3] = 
{
	{ .x = 11, .y = 1 },
	{ .x = 5, .y = 15 },
	{ .x = 14, .y = 18 }

};

void *p_sprites[4];

void initPointers()
{
	p_sprites[0] = &player;
	p_sprites[1] = &enemy;
	p_sprites[2] = &shoot;
	p_sprites[3] = &door;
	
}

void draw_tile(unsigned char x,unsigned char y, int type){

	switch(type){
		case 0://wall
			cpc_PutSp(sp_tile_brick,TILE_Y,TILE_X,cpc_GetScrAddress(x*TILE_X,y*TILE_Y));
		break;
		case 1://floor
			cpc_PutSp(sp_tile_floor,TILE_Y,TILE_X,cpc_GetScrAddress(x*TILE_X,y*TILE_Y));
		break;
		case 2://player
			player.address=cpc_GetScrAddress(x*TILE_X,y*TILE_Y);
			cpc_PutMaskSp(player.frame,player.height,player.width,player.address);
			player.x=x;
			player.y=y;
			player.ox=x;
			player.oy=y;
			player.actx = player.ox * TILE_X;
			player.acty = player.oy * TILE_Y;

		break;
		case 3://enemy

			enemy.address=cpc_GetScrAddress(x*TILE_X,y*TILE_Y);
			cpc_PutMaskSp(enemy.frame,enemy.height,enemy.width,enemy.address);
			enemy.x=x;
			enemy.y=y;
			enemy.ox=x;
			enemy.oy=y;
			enemy.actx= enemy.ox  * TILE_X;
			enemy.acty = enemy.oy * TILE_Y;
		break;
		case 4:
			door.address=cpc_GetScrAddress(x*TILE_X,y*TILE_Y);
			cpc_PutMaskSp(door.frame,door.height,door.width,door.address);
			door.x=x;
			door.y=y;
			door.ox=x;
			door.oy=y;
			door.actx=door.ox;
			door.acty=door.oy;
			mapa[x][y]=0;
		break;
			
	}

}

void draw_tilemap()
{
	unsigned char x,y;

	for(y=0;y<20;y++){
		for(x=0;x<20;x++){
			if(mapa[x][y]!=1){
				draw_tile(x, y, mapa[x][y]);
			}
		}
	}	
		
}

void set_colours(void)
{
	unsigned char x;
	for (x=0; x<16; x++)
	{
		cpc_SetInk(x,tintas[x]);
	}
	cpc_SetBorder(0);
}

void updatePlayer(struct sprite *pl){
	int old;
	if(pl->steps>=0){
		switch(pl->dir){
			case DIR_ARRIBA:
				pl->actx=pl->x*TILE_X;
				pl->acty=pl->y*TILE_Y+((pl->steps-1)*2);	
				old = cpc_GetScrAddress(pl->x*TILE_X,pl->y*TILE_Y+((pl->steps)*2));
			break;
			case DIR_ABAJO:
				pl->actx=pl->x*TILE_X;
				pl->acty=pl->y*TILE_Y-((pl->steps-1)*2);
				old = cpc_GetScrAddress(pl->x*TILE_X,pl->y*TILE_Y-((pl->steps)*2));
			break;
			case DIR_IZQ:
				pl->actx=pl->x*TILE_X+(pl->steps-1);
				pl->acty=pl->y*TILE_Y;
				old =cpc_GetScrAddress(pl->x*TILE_X+(pl->steps),pl->y*TILE_Y);
			break;
			case DIR_DER:
				pl->actx=pl->x*TILE_X-(pl->steps-1);
				pl->acty=pl->y*TILE_Y;
				old =cpc_GetScrAddress(pl->x*TILE_X-(pl->steps),pl->y*TILE_Y);
			break;
			default:
				pl->address =cpc_GetScrAddress(pl->x*TILE_X,pl->y*TILE_Y);
				old =cpc_GetScrAddress(pl->ox*TILE_X,pl->oy*TILE_Y);
				break;
			
		}
		
		if(pl->steps==1){
			pl->actx=pl->x*TILE_X;
			pl->acty=pl->y*TILE_Y;
		}
		
		pl->address =cpc_GetScrAddress(pl->actx,pl->acty);
		pl->anim_num=(pl->anim_num+1)%pl->modulo;
		
		if(pl->steps>0){
			pl->steps--;
		}else{
			pl->anim_num=0;
		}
		
		
		switch(pl->type){
			case PLAYER:
				pl->frame=pl_frames[pl->dir][pl->anim_num];
			break;
			case ENEMY:
				pl->frame=enemy_frames[pl->dir][pl->anim_num];
			break;
		}
			
			cpc_PutSp(sp_tile_floor, TILE_Y, TILE_X, old);
			cpc_PutMaskSp(pl->frame,pl->height, pl->width,pl->address);
	}
}

void updateShoot(struct sprite *sh)
{

		sh->actx=sh->x*TILE_X;
		sh->acty=sh->y*TILE_Y;
		
		sh->address =cpc_GetScrAddress(sh->actx,sh->acty);
			
		cpc_PutSp(sp_tile_floor, TILE_Y, TILE_X, cpc_GetScrAddress(sh->ox*TILE_X,sh->oy*TILE_Y));
		cpc_PutMaskSp(sh->frame,sh->height, sh->width,sh->address);
}


void asignKeys(void)
{
		cpc_AssignKey(IZQ,0x4820);
		cpc_AssignKey(DER,0x4720);
		cpc_AssignKey(ARRIBA,0x4708);
		cpc_AssignKey(ABAJO,0x4710);
		cpc_AssignKey(SPACE,0x4580);
		cpc_AssignKey(KEY_1,0x4801);
		cpc_AssignKey(KEY_ESC,0x4804);

		cpc_AssignKey(KEY_2, 0x4802);
		cpc_AssignKey(KEY_3, 0x4702);


}

void initGraph(int lvl)
{
	//char str[50];
	int src_enemy;
	int target_player;

	// First time we load the first graph
	
	if(lvl!=prevLevel)
		LoadGraph(lvl);
	// Get closest node for enemy and player
	src_enemy = GetClosestNode(lvl, enemy.x, enemy.y);
	target_player = GetClosestNode(lvl, player.x, player.y);
	
	// Get the path
	GetPath(src_enemy, target_player, 1);
	enemy.nodosRecorridos = 1; // siempre hacer esto al obtener un nuevo path	
}

void initData(void)
{
						
		player.frame=(unsigned char *)sp_player_abajo;
		player.x=7;
		player.y=4;
		player.type=PLAYER;
		player.shoot=0;
		player.ox=player.x;
		player.oy=player.y;
		player.actx=player.ox*TILE_X;
		player.acty=player.oy*TILE_Y;
		player.steps = 0;
		player.dir=DIR_ABAJO;
		player.anim_num=0;
		player.modulo=3;
		player.dir_prev=player.dir;
		player.height=TILE_Y;
		player.width=TILE_X;
		player.address=cpc_GetScrAddress(player.x,player.y);
		
		//frames player
		pl_frames[DIR_ABAJO][0]=(unsigned char *)sp_player_abajo;
		pl_frames[DIR_ABAJO][1]=(unsigned char *)sp_player_abajo2;
		pl_frames[DIR_ABAJO][2]=(unsigned char *)sp_player_abajo3;
		
		pl_frames[DIR_ARRIBA][0]=(unsigned char *)sp_player_arriba;
		pl_frames[DIR_ARRIBA][1]=(unsigned char *)sp_player_arriba2;
		pl_frames[DIR_ARRIBA][2]=(unsigned char *)sp_player_arriba3;
		
		pl_frames[DIR_DER][0]=(unsigned char *)sp_player_derecha;
		pl_frames[DIR_DER][1]=(unsigned char *)sp_player_derecha2;
		pl_frames[DIR_DER][2]=(unsigned char *)sp_player_derecha3;
		
		pl_frames[DIR_IZQ][0]=(unsigned char *)sp_player_izquierda;
		pl_frames[DIR_IZQ][1]=(unsigned char *)sp_player_izquierda2;
		pl_frames[DIR_IZQ][2]=(unsigned char *)sp_player_izquierda3;
		

		enemy.frame=(unsigned char *)sp_enemy_abajo;
		enemy.x=11;
		enemy.y=1;
		enemy.vida=3;
		enemy.type=ENEMY;
		enemy.ox=enemy.x;
		enemy.oy=enemy.y;
		enemy.actx=enemy.ox*TILE_X;
		enemy.acty=enemy.oy*TILE_Y;
		enemy.steps = 0;
		enemy.dir=DIR_ABAJO;
		enemy.dir_prev=enemy.dir;
		enemy.anim_num=0;
		enemy.modulo=3;
		enemy.height=TILE_Y;
		enemy.width=TILE_X;
		enemy.address=cpc_GetScrAddress(enemy.x,enemy.y);
		
		enemy.nodosRecorridos = 1;
		enemy.millas = 0;
		enemy.timestop = 0;
		
		enemyCont=SCORE_GOAL;

		//frames enemy
		enemy_frames[DIR_ABAJO][0]=(unsigned char *)sp_enemy_abajo;
		enemy_frames[DIR_ABAJO][1]=(unsigned char *)sp_enemy_abajo2;
		enemy_frames[DIR_ABAJO][2]=(unsigned char *)sp_enemy_abajo3;
		
		enemy_frames[DIR_ARRIBA][0]=(unsigned char *)sp_enemy_arriba;
		enemy_frames[DIR_ARRIBA][1]=(unsigned char *)sp_enemy_arriba2;
		enemy_frames[DIR_ARRIBA][2]=(unsigned char *)sp_enemy_arriba3;
		
		enemy_frames[DIR_DER][0]=(unsigned char *)sp_enemy_derecha;
		enemy_frames[DIR_DER][1]=(unsigned char *)sp_enemy_derecha2;
		enemy_frames[DIR_DER][2]=(unsigned char *)sp_enemy_derecha3;
		
		enemy_frames[DIR_IZQ][0]=(unsigned char *)sp_enemy_izquierda;
		enemy_frames[DIR_IZQ][1]=(unsigned char *)sp_enemy_izquierda2;
		enemy_frames[DIR_IZQ][2]=(unsigned char *)sp_enemy_izquierda3;
		
		shoot.frame=(unsigned char *)sp_disparo_vertical;
		shoot.x=0;
		shoot.y=0;
		shoot.type=SHOOT;
		shoot.shoot=0;
		shoot.ox=shoot.x;
		shoot.oy=shoot.y;
		shoot.actx=shoot.ox*TILE_X;
		shoot.acty=shoot.oy*TILE_Y;
		shoot.steps = 0;
		shoot.dir=DIR_ABAJO;
		shoot.anim_num=0;
		shoot.modulo=1;
		shoot.dir_prev=shoot.dir;
		shoot.height=TILE_Y;
		shoot.width=TILE_X;
		shoot.address=cpc_GetScrAddress(shoot.x,shoot.y);
		
		
		door.frame=(unsigned char *)sp_door;
		door.x=0;
		door.y=0;
		door.vida=10;
		door.ox=door.x;
		door.oy=door.y;
		door.actx=door.ox*TILE_X;
		door.acty=door.oy*TILE_Y;
		door.steps = 50;
		door.dir=DIR_IZQ;
		door.dir_prev=door.dir;
		door.anim_num=0;
		door.modulo=3;
		door.height=TILE_Y;
		door.width=TILE_X;
		door.address=cpc_GetScrAddress(door.x,door.y);
		
		initGraph(actualLevel);
		
}

/* 1 - MENU */ 
char Menu()
{
	char choice = -1;
	
	
	cpc_WyzInitPlayer(SOUND_TABLE_MENU, RULE_TABLE_MENU, EFFECT_TABLE, SONG_TABLE_MENU);		//initialize data

	cpc_WyzLoadSong(0);	//Select song to play (uncompress it and the start to play)
	cpc_WyzSetPlayerOn();
	cpc_ClrScr();


	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("CHIPOTLE", 30, 1 * 16);
	cpc_PrintGphStrXY(";WARFARE", 30, 2 * 16);

	cpc_SetInkGphStr(0, 0);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("1;JUGAR", 29, 5 * 16);
	cpc_PrintGphStrXY("2;COMO;JUGAR", 29, 6 * 16);
	cpc_PrintGphStrXY("ESC;SALIR", 29, 7 * 16);

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("F;2014;COSMIC;WARPIGS", 18, 10 * 16);

	while (choice == -1)
	{
		cpc_ScanKeyboard();
		if (cpc_TestKeyF(KEY_1) == 1)
			choice = GAME_STATE_INTRO;
		if (cpc_TestKeyF(KEY_2) == 1)
			choice = GAME_STATE_HOWTO;
		if (cpc_TestKeyF(KEY_ESC) == 1)
			choice = GAME_STATE_EXIT;
		if (cpc_TestKeyF(KEY_3) == 1)
			choice = GAME_STATE_CREDITS;
	}
	while (cpc_AnyKeyPressed());

	if (choice != 3)
		cpc_WyzSetPlayerOff();

	return choice;

}

/* 2 - COMO JUGAR */
char howTo()
{
	char choice = -1;
	cpc_WyzInitPlayer(SOUND_TABLE_MENU, RULE_TABLE_MENU, EFFECT_TABLE, SONG_TABLE_MENU);		//initialize data

	cpc_WyzLoadSong(0);	//Select song to play (uncompress it and the start to play)
	cpc_WyzSetPlayerOn();

	cpc_ClrScr();

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 42);
	cpc_SetInkGphStr(2, 42);
	cpc_SetInkGphStr(3, 42);

	cpc_PrintGphStrXY(";COMO;;", 30, 1 * 16);
	cpc_PrintGphStrXY(";JUGAR;", 30, 2 * 16);

	cpc_SetInkGphStr(0, 0);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("MOVIMIENTO", 28, 4 * 16);
	cpc_PrintGphStrXY("W;-;ARRIBA", 28, 5 * 16);
	cpc_PrintGphStrXY("A;-;IZQUIERDA", 28, 6 * 16);
	cpc_PrintGphStrXY("S;-;ABAJO", 28, 7 * 16);
	cpc_PrintGphStrXY("D;-;DERECHA", 28, 8 * 16);

	cpc_PrintGphStrXY("DISPARAR;-;ESPACIO", 23, 10 * 16);
	cpc_PrintGphStrXY("TIENES;QUE;MATAR;A;LOS;MARIACHIS;Y", 0, 10 * 16);
	cpc_PrintGphStrXY("LUEGO;DISPARAR;A;LA;PUERTA;PARA;GANAR", 0, 11 * 16);

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("PULSA;ESC;PARA;VOLVER", 18, 12 * 16);

	while (choice == -1)
	{
		cpc_ScanKeyboard();
		if (cpc_TestKeyF(KEY_ESC) == 1)
			choice = GAME_STATE_MENU;
	}
	while (cpc_AnyKeyPressed());

	cpc_WyzSetPlayerOff();

	return choice;
}

/* 2 - "CINEMATICA" INTRODUCCION */
char introduccion()
{
	char choice = -1;
	int jugar = 0;

	cpc_WyzInitPlayer(SOUND_TABLE_MENU, RULE_TABLE_MENU, EFFECT_TABLE, SONG_TABLE_MENU);		//initialize data

	cpc_WyzLoadSong(0);	//Select song to play (uncompress it and the start to play)
	cpc_WyzSetPlayerOn();

	cpc_ClrScr();


	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 42);
	cpc_SetInkGphStr(2, 42);
	cpc_SetInkGphStr(3, 42);

	cpc_PrintGphStrXY(";YEAR;2016;", 30, 1 * 16);

	cpc_SetInkGphStr(0, 0);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("LOS;MARIACHIS;HAN;CONQUISTADO;LA;TIERRA;", 0, 2 * 16);
	cpc_PrintGphStrXY("EL;PAN;HA;SIDO;PERSEGUIDO;Y;ANIQUILADO", 0, 3 * 16);
	cpc_PrintGphStrXY("Y;LOS;TACOS;SON;AHORA;EL;NUEVO;PAN.", 0, 4 * 16);
	cpc_PrintGphStrXY("LA;GENTE;TIENE;MUCHOS;GASES;POR;COMER", 0, 5 * 16);
	cpc_PrintGphStrXY("TANTO;TACO;Y;PROBABLEMENTE;SEA;EL;FIN;DE", 0, 6 * 16);
	cpc_PrintGphStrXY("LA;HUMANIDAD;TAL;Y;COMO;LA;CONOCEMOS.", 0, 7 * 16);
	cpc_PrintGphStrXY("AQUELLOS;QUE;SOBREVIVEN;PASAN;A;FORMAR", 0, 8 * 16);
	cpc_PrintGphStrXY("PARTE;DE;LOS;MARIACHIS;DEL;NUEVO;MUNDO", 0, 9 * 16);
	cpc_PrintGphStrXY("O;COMO;SE;HACEN;LLAMAR;ELLOS;", 0, 10 * 16);
	cpc_PrintGphStrXY("LA;MARIACHIDAD.;", 0, 11 * 16);

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("PULSA;ESPACIO;PARA;CONTINUAR", 18, 12 * 16);


	while (choice == -1)
	{
		cpc_ScanKeyboard();
		if (cpc_TestKeyF(SPACE) == 1)
		{
			if (jugar == 0)
			{
				cpc_ClrScr();
				jugar = 1;

				cpc_SetInkGphStr(0, 0);
				cpc_SetInkGphStr(1, 170);
				cpc_SetInkGphStr(2, 170);
				cpc_SetInkGphStr(3, 170);

				cpc_PrintGphStrXY("LA;UNICA;ESPERANZA;DE;SUPERVIVENCIA", 0, 1 * 16);
				cpc_PrintGphStrXY("PARA;LA;GENTE;CORRIENTE;RESIDE;EN;UN", 0, 2 * 16);
				cpc_PrintGphStrXY("TEMPLO;ANTIGUO;DEL;PAN;EN;EL;QUE;SE;DICE", 0, 3 * 16);
				cpc_PrintGphStrXY("QUE;SE;ENCUENTRA;EL;PAN;LEGENDARIO;QUE", 0, 4 * 16);
				cpc_PrintGphStrXY("UTILIZO;JESUCRISTO;PARA;MULTIPLICARLO", 0, 5 * 16);
				cpc_PrintGphStrXY("Y;MANTIENE;PROPIEDADES;MULTIPLICADORAS.", 0, 6 * 16);
				cpc_PrintGphStrXY("", 0, 7 * 16);
				cpc_PrintGphStrXY("MANHOLO;EL;HUMANO;HA;INTERCEPTADO;LAS", 0, 8 * 16);
				cpc_PrintGphStrXY("COMUNICACIONES;MARIACHIS;DE;LOCALIZAZION", 0, 9 * 16);
				cpc_PrintGphStrXY("DEL;PAN;Y;SE;DIRIGE;A;CONSEGUIRLO;PARA", 0, 10 * 16);
				cpc_PrintGphStrXY("SALVARNOS;A;TODOS.", 0, 11 * 16);


				cpc_SetInkGphStr(0, 2);
				cpc_SetInkGphStr(1, 170);
				cpc_SetInkGphStr(2, 170);
				cpc_SetInkGphStr(3, 170);

				cpc_PrintGphStrXY("PULSA;ESPACIO;PARA;CONTINUAR", 18, 12 * 16);

			}
			else if(jugar==1)
			{
				cpc_ClrScr();
				jugar = 2;
				
				cpc_SetInkGphStr(0, 0);
				cpc_SetInkGphStr(1, 170);
				cpc_SetInkGphStr(2, 170);
				cpc_SetInkGphStr(3, 170);

				cpc_PrintGphStrXY("ATENCION", 0, 1 * 16);
				cpc_PrintGphStrXY("", 0, 2 * 16);
				cpc_PrintGphStrXY("LOS;MARIACHIS;TIENEN;PODERES;ESPECIALES", 0, 3 * 16);
				cpc_PrintGphStrXY("DE;TACOS;QUE;ALTERAN;EL;ESPACIO;TIEMPO;Y", 0, 4 * 16);
				cpc_PrintGphStrXY("TE;BLOQUEARAN;TUS;ACCIONES;QUIZAS;CUANDO", 0, 5 * 16);
				cpc_PrintGphStrXY("JUEGUES;", 0, 6 * 16);
				cpc_PrintGphStrXY("", 0, 7 * 16);
				cpc_PrintGphStrXY("SI;ESO;TE;PASA;TIENES;QUE;PULSAR;ESC;Y;", 0, 8 * 16);
				cpc_PrintGphStrXY("VOLVER;A;INTENTARLO;PORQUE;LOS;TACOS;SON", 0, 9 * 16);
				cpc_PrintGphStrXY("ASI;DE;PODEROSOS;AUNQUE;NO;LOS;VEAS", 0, 10 * 16);


				cpc_SetInkGphStr(0, 2);
				cpc_SetInkGphStr(1, 170);
				cpc_SetInkGphStr(2, 170);
				cpc_SetInkGphStr(3, 170);

				cpc_PrintGphStrXY("PULSA;ESPACIO;PARA;JUGAR", 18, 12 * 16);
			}
			else
			{
				choice = GAME_STATE_GAME;
			}
		}
	}
	while (cpc_AnyKeyPressed());

	cpc_WyzSetPlayerOff();

	return choice;
}

/* 3 - CREDITOS DEL JUEGO */
char creditos()
{
	char choice = -1;

	cpc_WyzInitPlayer(SOUND_TABLE_MENU, RULE_TABLE_MENU, EFFECT_TABLE, SONG_TABLE_MENU);		//initialize data

	cpc_WyzLoadSong(0);	//Select song to play (uncompress it and the start to play)
	cpc_WyzSetPlayerOn();

	cpc_ClrScr();

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("CREDITOS;DE;JUEGO", 20, 1 * 16);

	cpc_SetInkGphStr(0, 0);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("GAME;DESIGN;BY;COSMIC;WARPIGS", 10, 3 * 16);
	cpc_PrintGphStrXY("MUSIC;DESIGN;BY;COSMIC;WARPIGS", 10, 4 * 16);
	cpc_PrintGphStrXY("GAME;PROGRAMMING;BY;COSMIC;WARPIGS", 10, 5 * 16);
	cpc_PrintGphStrXY("SOUND;EFFECTS;BY;MOJON;TWINS", 10, 6 * 16);

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("GRACIAS;POR;JUGAR;DALE;A;LIKE", 10, 10 * 16);
	cpc_PrintGphStrXY("", 18, 11 * 16);
	cpc_PrintGphStrXY("PULSA;ESPACIO;PARA;SALIR", 15, 12 * 16);

	while (choice == -1)
	{
		cpc_ScanKeyboard();
		if (cpc_TestKeyF(SPACE) == 1)
			choice = GAME_STATE_MENU;
	}
	while (cpc_AnyKeyPressed());

	//if (choice != 3)
		cpc_WyzSetPlayerOff();

	return choice;
}

/*7 - Lose*/
char Lose()
{
	char choice = -1;


	cpc_WyzInitPlayer(SOUND_TABLE_MENU, RULE_TABLE_MENU, EFFECT_TABLE, SONG_TABLE_MENU);		//initialize data

	cpc_WyzLoadSong(0);	//Select song to play (uncompress it and the start to play)
	cpc_WyzSetPlayerOn();
	cpc_ClrScr();

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("SALUDOS;MARIACHIDAD;LA;TIERRA;TE;SALUDA", 0, 1 * 16);

	cpc_SetInkGphStr(0, 0);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("LAS;TROPAS;MARIACHIS;LOGRARON;DETENER;A", 0, 2 * 16);
	cpc_PrintGphStrXY("MANHOLO;Y;LA;HUMANIDAD;PERDIO;FINALMENTE", 0, 3 * 16);
	cpc_PrintGphStrXY("", 0, 4 * 16);
	cpc_PrintGphStrXY("EL;PAN;DE;CRISTO;HA;SIDO;TRANSFORMADO;", 0, 5 * 16);
	cpc_PrintGphStrXY("CON;TECNICAS;MARIACHIS;Y;AHORA;ES;EL", 0, 6 * 16);
	cpc_PrintGphStrXY("TACO;DE;JESUCRISTO;MARIACHI", 0, 7 * 16);
	cpc_PrintGphStrXY("", 0, 8 * 16);
	cpc_PrintGphStrXY("ES;EL;ANO;2016;Y;LA;MARIACHIDAD;ESTA;EN", 0, 9 * 16);
	cpc_PrintGphStrXY("PLENA;VIA;DE;EXPANSION;POR;EL;UNIVERSO", 0, 10 * 16);
	cpc_PrintGphStrXY("EL;MENU;DE;TODOS;LOS;DIAS;ES;TACOS", 0, 11 * 16);

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("PULSA;1;PARA;CONTINUAR;Y;2;PARA;SALIR", 0, 12 * 16);


	while (choice == -1)
	{
		cpc_ScanKeyboard();
		if (cpc_TestKeyF(KEY_1) == 1)
		{
			choice = GAME_STATE_GAME;
		}
		if (cpc_TestKeyF(KEY_2) == 1)
		{
			choice = GAME_STATE_MENU;
		}
	}
	while (cpc_AnyKeyPressed());

	cpc_WyzSetPlayerOff();

	return choice;
}

/*7 - Win*/
char Win()
{
	char choice = -1;

	cpc_WyzInitPlayer(SOUND_TABLE_MENU, RULE_TABLE_MENU, EFFECT_TABLE, SONG_TABLE_MENU);		//initialize data

	cpc_WyzLoadSong(0);	//Select song to play (uncompress it and the start to play)
	cpc_WyzSetPlayerOn();
	cpc_ClrScr();



	cpc_SetInkGphStr(0, 0);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("FINALMENTE;LA;FALTA;DE;TIEMPO;ANTE;LA", 0, 1 * 16);
	cpc_PrintGphStrXY("ENTREGA;PARA;EL;CONCURSO;AYUDO;A;MANHOLO", 0, 2 * 16);
	cpc_PrintGphStrXY("EN;SECRETO;EN;SU;AVENTURA;SIN;PARANGON", 0, 3 * 16);
	cpc_PrintGphStrXY("", 0, 4 * 16);
	cpc_PrintGphStrXY("MANHOLO;CONSIGUIO;EL;PAN;LEGENDARIO;DE", 0, 5 * 16);
	cpc_PrintGphStrXY("JESUCRISTO;Y;LA;HUMANIDAD;SE;HA;SALVO", 0, 6 * 16);
	cpc_PrintGphStrXY("", 0, 7 * 16);
	cpc_PrintGphStrXY("LOS;MARIACHIS;RESTANTES;VUELVEN;A;TOCAR;", 0, 8 * 16);
	cpc_PrintGphStrXY("CANCIONES;Y;ESPERAR;SU;PROXIMO;MOMENTO;Y", 0, 9 * 16);
	cpc_PrintGphStrXY("MANHOLO;HA;FUNDADO;UNA;EMPRESA;ESPACIAL", 0, 10 * 16);
	cpc_PrintGphStrXY("FIN", 0, 11 * 16);

	cpc_SetInkGphStr(0, 2);
	cpc_SetInkGphStr(1, 170);
	cpc_SetInkGphStr(2, 170);
	cpc_SetInkGphStr(3, 170);

	cpc_PrintGphStrXY("PULSA;1;PARA;CONTINUAR", 10, 12 * 16);


	while (choice == -1)
	{
		cpc_ScanKeyboard();
		if (cpc_TestKeyF(KEY_1) == 1)
		{
			choice = GAME_STATE_CREDITS;
		}
	}
	while (cpc_AnyKeyPressed());

	cpc_WyzSetPlayerOff();

	return choice;
}


unsigned char collide(struct sprite *sp1, struct sprite *sp2)
{
  return !(
	sp1->actx > sp2->actx+sp2->width
	|| sp2->actx > sp1->actx+sp1->width
	|| sp1->acty > sp2->acty+sp2->height
	|| sp2->acty > sp1->acty+sp1->height);
}

void applySound(int sound)
{
	if ((cpc_WyzTestPlayer() & 8) == 0) 
		cpc_WyzStartEffect(2,sound);
}


void crearDisparo(struct sprite *pl)
{
	switch(pl->dir){
		case DIR_ARRIBA:
			if(mapa[pl->x][pl->y-1]!=0){
				pl->shoot = 1;
				shoot.x=pl->x;
				shoot.y=pl->y-1;
				shoot.ox=shoot.x;
				shoot.oy=shoot.y;
				shoot.actx=shoot.ox*TILE_X;
				shoot.acty=shoot.oy*TILE_Y;
				shoot.dir=DIR_ARRIBA;
				shoot.dir_prev=-1;
				shoot.steps=TILE_Y/2;
				shoot.frame=(unsigned char *)sp_disparo_vertical;
			}else if(pl->x==door.x && pl->y-1==door.y){
				pl->shoot = 1;
				shoot.shoot=1;
				shoot.dir=DIR_ARRIBA;
				shoot.dir_prev=shoot.dir;
			}
		break;
		case DIR_ABAJO:
			if(mapa[pl->x][pl->y+1]!=0){
				pl->shoot = 1;
				shoot.x=pl->x;
				shoot.y=pl->y+1;
				shoot.ox=shoot.x;
				shoot.oy=shoot.y;
				shoot.actx=shoot.ox*TILE_X;
				shoot.acty=shoot.oy*TILE_Y;
				shoot.dir=DIR_ABAJO;
				shoot.dir_prev=-1;
				shoot.steps=TILE_Y/2;
				shoot.frame=(unsigned char *)sp_disparo_vertical;
			}else if(pl->x==door.x && pl->y+1==door.y){
				pl->shoot = 1;
				shoot.shoot=1;
				shoot.dir=DIR_ABAJO;
				shoot.dir_prev=shoot.dir;
			}
		break;
		case DIR_DER:
			if(mapa[pl->x+1][pl->y]!=0){
				pl->shoot = 1;
				shoot.x=pl->x+1;
				shoot.y=pl->y;
				shoot.ox=shoot.x;
				shoot.oy=shoot.y;
				shoot.actx=shoot.ox*TILE_X;
				shoot.acty=shoot.oy*TILE_Y;
				shoot.dir=DIR_DER;
				shoot.dir_prev=-1;
				shoot.steps=TILE_X;
				shoot.frame=(unsigned char *)sp_disparo_horizontal;
			}else if(pl->x+1==door.x && pl->y==door.y){
				pl->shoot = 1;
				shoot.shoot=1;
				shoot.dir=DIR_DER;
				shoot.dir_prev=shoot.dir;
			}
		break;
		case DIR_IZQ:
			if(mapa[pl->x-1][pl->y]!=0){
				pl->shoot = 1;
				shoot.x=pl->x-1;
				shoot.y=pl->y;
				shoot.ox=shoot.x;
				shoot.oy=shoot.y;
				shoot.actx=shoot.ox*TILE_X;
				shoot.acty=shoot.oy*TILE_Y;
				shoot.dir=DIR_IZQ;
				shoot.dir_prev=-1;
				shoot.steps=TILE_X;
				shoot.frame=(unsigned char *)sp_disparo_horizontal;
			}else if(pl->x-1==door.x && pl->y==door.y){
				pl->shoot = 1;
				shoot.shoot=1;
				shoot.dir=DIR_IZQ;
				shoot.dir_prev=shoot.dir;
			}
		break;
	}

	if (pl->shoot == 1)
		applySound(SOUND_SHOOT);
}

void eliminarDisparo()
{
	player.shoot=0;
	cpc_PutSp(sp_tile_floor,TILE_Y,TILE_X,cpc_GetScrAddress(shoot.actx,shoot.acty));

	applySound(SOUND_SHOOT_END);
	
}

void resetDoor()
{
	cpc_PutSp(sp_tile_floor,TILE_Y,TILE_X,door.address);
	mapa[door.x][door.y]=4;
}



void buscarJugador(struct sprite *en)
{

	if(en->x==player.x)//para eje y
	{
			if(player.y < en->y)
				en->dir=DIR_ARRIBA;
			else
				en->dir=DIR_ABAJO;
	}
	else if(en->y == player.y) // para eje x
	{
			if(player.x < en->x)
				en->dir=DIR_IZQ;
			else
				en->dir=DIR_DER;
	}
	else // dir == STOP
	{
		en->dir == STOP;
	}
}

void updateMirada(char lvl, struct sprite * en)
{
	// Calcular la direcion del mov
	if(pathEnemy1[en->nodosRecorridos]!=-1 ) // hay mas de un nodo en el path
	{
		if(Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].x == en->x)
			if(Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].y < en->y)
				en->dir=DIR_ARRIBA;
			else
				en->dir=DIR_ABAJO;
		else // para eje x
			if(Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].y == en->y)						
				if(Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].x < en->x)
					en->dir=DIR_IZQ;
				else
					en->dir=DIR_DER;
	}
	else//esta en un nodo -1
	{
		en->dir=STOP;
	}


}
void updatePathEnemy(struct sprite * en)
{
	char src_enemy;
	char target_player;
	src_enemy = GetClosestNode(actualLevel, en->x, en->y);
	target_player = GetClosestNode(actualLevel, player.x, player.y);

	GetPath(src_enemy, target_player, 1);

	if (pathEnemy1[1] != -1)
		en->nodosRecorridos = 1;
	/*else
	{
		en->nodosRecorridos = 0;
		updateMirada(actualLevel, en);
	}*/

}

void updateEnemy(struct sprite *en)
{

	if (en->millas >= MILLASMAX)
	{
		en->timestop++;
		if (en->timestop >= TIMEMAX)
		{
			en->millas = 0;
			en->timestop = 0;
		}
	}
	else
	{
		// Si ha llegado al next node o es -1 
		if ((Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].x == en->x && Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].y == en->y) ||
			pathEnemy1[en->nodosRecorridos] == -1 || en->dir == STOP)
		{
			// ha llegado al siguiente
			if (Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].x == en->x && Lvl1Nodes[pathEnemy1[en->nodosRecorridos]].y == en->y)
			{
				// Si el next node (nodosRecorridos+1) es -1 calculo path
				if (pathEnemy1[en->nodosRecorridos + 1] == -1)
				{
					// Calculo nuevo path
					updatePathEnemy(en);

				}
				else
				{
					// Paso de nodo
					en->nodosRecorridos++;
				}

			}
			else // es -1 y toca actualzar
			{
				// Actualizar Path
				updatePathEnemy(en);

			}

			updateMirada(actualLevel, en);

		}

		if (en->dir == STOP)
		{
			buscarJugador(en);
		}

		switch (en->dir)
		{
			case DIR_IZQ:
				if (mapa[en->x - 1][en->y] != 0)
				{
					en->ox = en->x; en->x--; en->oy = en->y; en->steps = TILE_X;
					en->millas++;
				}
				break;
			case DIR_DER:
				if (mapa[en->x + 1][en->y] != 0)
				{
					en->ox = en->x; en->x++; en->oy = en->y; en->steps = TILE_X;
					en->millas++;
				}
				break;
			case DIR_ARRIBA:
				if (mapa[en->x][en->y - 1] != 0)
				{
					en->oy = en->y; en->y--; en->ox = en->x; en->steps = TILE_Y / 2;
					en->millas++;
				}
				break;
			case DIR_ABAJO:
				if (mapa[en->x][en->y + 1] != 0)
				{
					en->oy = en->y; en->y++; en->ox = en->x; en->steps = TILE_Y / 2;
					en->millas++;
				}
				break;
		}
			
	}
	
}

void drawHud()
{
	char str[10];
	
	if(enemyCont>9)
		sprintf(str,"ENEMY;%d",enemyCont);
	else
		sprintf(str,"ENEMY;0%d",enemyCont);

	cpc_PrintGphStrXY(str,    0, 0);
}

void resetEnemy(struct sprite *en)
{
		char r;
		r = rand() % 3;
		
	
		cpc_PutSp(sp_tile_floor,TILE_Y,TILE_X,en->address);

		en->x=nodosRespawn[r].x;
		en->y=nodosRespawn[r].y;
		en->vida=3;
		en->ox=en->x;
		en->oy=en->y;
		en->actx=en->ox*TILE_X;
		en->acty=en->oy*TILE_Y;
		en->steps = 0;
		en->millas = 0;
		en->timestop = 0;
		
		updatePathEnemy(en);
		updateMirada(actualLevel,en);
		
		if(en->dir==STOP){
			buscarJugador(en);
		}
		
		applySound(SOUND_DEATH);
		enemyCont--;
		if(enemyCont==0)
			resetDoor();
		drawHud();
		
}




/* 2 - GAME */
char Game()
{
	
	char choice = state;
	char pantalla = 0;
	int elapsedFrames = 0;
	cpc_WyzInitPlayer(SOUND_TABLE_MENU, RULE_TABLE_MENU, EFFECT_TABLE, SONG_TABLE_JUEGO);		//initialize data
	cpc_WyzLoadSong(0);	//Select song to play (uncompress it and the start to play)
	cpc_WyzSetPlayerOn();
	cpc_ClrScr();
	initData();
		
	draw_tilemap();
	drawHud();
	while (choice == GAME_STATE_GAME)
	{
			if (cpc_TestKey(KEY_ESC)==1) 
			{
			  choice = GAME_STATE_MENU;
			  prevLevel=actualLevel;
			  mapa[door.x][door.y]=4;			  
			}
			else // juego
			{
				//si no se est animando
				if(player.steps==0){
					//comprueba la entrada de teclado
					if(cpc_TestKey(SPACE)==1 && player.shoot==0){crearDisparo(p_sprites[0]);}
					if (cpc_TestKey(DER)==1 && player.x<20 && mapa[player.x+1][player.y]!=0){ player.ox= player.x; player.x++; player.oy=player.y; player.dir=DIR_DER; player.steps = TILE_X;}
					else if (cpc_TestKey(IZQ)==1 && player.x>0 && mapa[player.x-1][player.y]!=0){player.ox=player.x; player.x--; player.oy=player.y; player.dir=DIR_IZQ; player.steps = TILE_X;}
					else if (cpc_TestKey(ARRIBA)==1 && player.y>0 && mapa[player.x][player.y-1]!=0){player.oy=player.y; player.y--; player.ox=player.x; player.dir=DIR_ARRIBA; player.steps = TILE_Y/2;}
					else if (cpc_TestKey(ABAJO)==1 && player.y<20 && mapa[player.x][player.y+1]!=0){ player.oy=player.y; player.y++; player.ox=player.x; player.dir=DIR_ABAJO; player.steps = TILE_Y/2;}

				}
						
				if (enemyCont>0 && (enemy.steps == 0 || enemy.millas >= MILLASMAX))
					updateEnemy(p_sprites[1]);
				

				if (elapsedFrames++ >= ELAPSED_TIME) {
						if(player.shoot==1){
							if(shoot.dir==shoot.dir_prev){
								switch(shoot.dir){
									case DIR_ARRIBA:
										if(mapa[shoot.x][shoot.y-1]!=0 && shoot.y>0){
											shoot.oy=shoot.y;
											shoot.y--;
											shoot.ox=shoot.x;
											shoot.steps=TILE_Y/2;
										
										}else{
											eliminarDisparo();
										}
									break;
									case DIR_ABAJO:
										if(mapa[shoot.x][shoot.y+1]!=0 && shoot.y<19){
											shoot.oy=shoot.y;
											shoot.y++;
											shoot.ox=shoot.x;
											shoot.steps=TILE_Y/2;
										}else
										{
											eliminarDisparo();
										}
									break;
									case DIR_DER:
										if (mapa[shoot.x + 1][shoot.y] != 0 && shoot.x < 19){
											shoot.ox = shoot.x;
											shoot.x++;
											shoot.oy = shoot.y;
											shoot.steps = TILE_X;
										}
										else{
											eliminarDisparo();
										}
									break;
									case DIR_IZQ:
										if(mapa[shoot.x-1][shoot.y]!=0 && shoot.x>0){
											shoot.ox=shoot.x;
											shoot.x--;
											shoot.oy=shoot.y;
											shoot.steps=TILE_X;
										}else
										{
											eliminarDisparo();
										}
									break;
								}
							}else{
								shoot.dir_prev=shoot.dir;
							}
						}
				
						if(player.steps!=0){
							updatePlayer(p_sprites[0]);
						}
						
						if(player.shoot==1){
							updateShoot(p_sprites[2]);
						}
						
						if(enemy.steps!=0){
							updatePlayer(p_sprites[1]);
						}

				elapsedFrames=0;
				}
				
				//colision con enemigo
				if(enemyCont>0)
				{
					if(collide(p_sprites[0],p_sprites[1]))
					{
						player.vida--;
						if(player.vida==0)
						{
							choice = GAME_STATE_LOSE;
							mapa[door.x][door.y]=4;
							prevLevel = actualLevel;
							
						}
						
					}
					//colision enemigo disparo
					if(player.shoot==1 && collide(p_sprites[2],p_sprites[1]) && ((shoot.x==enemy.x && shoot.y==enemy.y) || (shoot.ox==enemy.x && shoot.oy==enemy.y) || (shoot.x==enemy.ox && shoot.y==enemy.oy)) ){
						enemy.vida--;
						eliminarDisparo();
						if(enemy.vida==0){
							resetEnemy(p_sprites[1]);
						}
					}
				}
				
				
				if(enemyCont==0)
				{
					if(player.x == door.x && player.y==player.y)
					{
						choice = GAME_STATE_WIN;
						prevLevel=actualLevel;
						mapa[door.x][door.y]=4;
						
					}
				}
				
		}
	}

	cpc_WyzSetPlayerOff();
	return choice;
}



int main()
{
	initPointers();

	set_colours();
	cpc_SetBorder(0);
	cpc_SetModo(0);
	asignKeys();

	cpc_DisableFirmware();
	actualLevel=1;
	prevLevel=0;
	state = GAME_STATE_INITIAL;
	while(state != GAME_STATE_EXIT)
	{
	   switch (state)
	   {
		   case GAME_STATE_MENU:
			state = Menu();
			break;
		  case GAME_STATE_GAME:
			state = Game();
			break;
			
		  case GAME_STATE_HOWTO:
			  state = howTo();
			  break;

		  case GAME_STATE_INTRO:
			  state = introduccion();
			  break;
		  case GAME_STATE_CREDITS:
			  state = creditos();
			  break;	  
		  case GAME_STATE_WIN:
				state = Win();
				break;			
		  case GAME_STATE_LOSE:
			state = Lose();
		  break;
			
		  default:
			state = GAME_STATE_EXIT;
			break;
		}
	}

	return 0;

}